(function ($) {
  /**
   * Disable the continue buttons in the checkout process once they are clicked
   * and provide a notification to the user.
   */
  Drupal.behaviors.commerceStripePiWebPaymentRequest = {
    attach: function (context, settings) {
      /**
       * Detect what browser we're on.
       */
      var browser = (function(agent){
        switch(true){
          case agent.indexOf("edge") > -1: return "ie";
          case agent.indexOf("msie") > -1: return "ie";
          case agent.indexOf("opr") > -1 && !!window.opr: return "opera";
          case agent.indexOf("chrome") > -1 && !!window.chrome: return "chrome";
          case agent.indexOf("trident") > -1: return "ie";
          case agent.indexOf("firefox") > -1: return "firefox";
          case agent.indexOf("safari") > -1: return "safari";
          default: return "other";
        }
      })(window.navigator.userAgent.toLowerCase());

      //Hide label for commerce payment request checkout.
      // Init stripe once.
      $('body', context).once('commerce-payment-request-stripe', function () {
        // Change payment method depending on what is available. We default
        // to Google Pay.
        // Apple Pay.
        if (window.ApplePaySession) {
          $('#edit-commerce-payment-request label[for="edit-commerce-payment-payment-method-commerce-stripe-pi-payment-requestcommerce-payment-commerce-stripe-pi-payment-request"]').text('Apple Pay');
        }
        // Microsoft Pay.
        else if (browser == 'ie') {
          $('#edit-commerce-payment-request label[for="edit-commerce-payment-payment-method-commerce-stripe-pi-payment-requestcommerce-payment-commerce-stripe-pi-payment-request"]').text('Microsoft Pay');
        }
      });

      $('#stripe-pi-payment-request-button', context).once('commerce-payment-request-stripe').click(function (ev) {
        ev.preventDefault();
      });

      // Check the availability of the Payment Request API first.
      var activePaymentType = $('input[name="commerce_payment[payment_method]"]:checked', context).val();
      if (typeof activePaymentType !== 'undefined') {
        var $form = $(Drupal.settings.checkout_form_id);
//        console.log($form);
        console.log(activePaymentType);

        // Check if Stripe Payment Request tab active.
        if (activePaymentType.indexOf('commerce_stripe_pi_payment_request') >= 0) {
          // Create our Payment Request button.
          var stripe = Stripe(Drupal.settings.commerce_stripe_pi_payment_request.stripe_key);
          var paymentRequest = stripe.paymentRequest(Drupal.settings.commerce_stripe_pi_payment_request.paymentRequest);

          $form.once('commerce-payment-request-stripe', function () {
            var elements = stripe.elements();
            var prButton = elements.create('paymentRequestButton', {
              paymentRequest: paymentRequest,
              style: {
                paymentRequestButton: {
                  height: '44px',
                },
              },
            });

            // Check if we can make payments.
            paymentRequest.canMakePayment().then(function (result) {
              // If we can make Stripe Apple/Google payments, show the button.
              if (result) {
                Drupal.behaviors.commerceStripePiWebPaymentRequest.showButton();
                prButton.mount('#stripe-pi-payment-request-button');
              }
              // Else, if they can't make payments, show the setup info.
              else {
                Drupal.behaviors.commerceStripePiWebPaymentRequest.showSetUpButton();
              }
            });

            // Listen for a token, process and complete the payment on our end.
            paymentRequest.on('paymentmethod', function (ev) {
              // Send the token to the server to charge it!
              // Prepare data for backend call.
              var payment_intent_data = {
                payment_method_remote_id: ev.paymentMethod.id,
                result: ev,
                payment_request: Drupal.settings.commerce_stripe_pi_payment_request.paymentRequest,
              };

              // Get shipping details and pass to Stripe.
              var name = Drupal.behaviors.commerceStripePiWebPaymentRequest.getAddressData(Drupal.settings.commerce_stripe_pi_payment_request_address.name);
              var phone = Drupal.behaviors.commerceStripePiWebPaymentRequest.getAddressData(Drupal.settings.commerce_stripe_pi_payment_request_address.phone);
              var country = Drupal.behaviors.commerceStripePiWebPaymentRequest.getAddressData(Drupal.settings.commerce_stripe_pi_payment_request_address.address_country);
              var line1 = Drupal.behaviors.commerceStripePiWebPaymentRequest.getAddressData(Drupal.settings.commerce_stripe_pi_payment_request_address.address_line1);
              var line2 = Drupal.behaviors.commerceStripePiWebPaymentRequest.getAddressData(Drupal.settings.commerce_stripe_pi_payment_request_address.address_line2);
              var city = Drupal.behaviors.commerceStripePiWebPaymentRequest.getAddressData(Drupal.settings.commerce_stripe_pi_payment_request_address.address_city);
              var state = Drupal.behaviors.commerceStripePiWebPaymentRequest.getAddressData(Drupal.settings.commerce_stripe_pi_payment_request_address.address_state);
              var postal_code = Drupal.behaviors.commerceStripePiWebPaymentRequest.getAddressData(Drupal.settings.commerce_stripe_pi_payment_request_address.address_zip);

              function handleAction(response, payment_intent_path, ev) {
                var data = {
                  payment_method: response.payment_method_remote_id,
                  shipping: {
                    name: name,
                    phone:phone,
                    address: {
                      line1: line1,
                      line2: line2,
                      city: city,
                      postal_code: postal_code,
                      state: state,
                      country: country
                    },
                  }
                };
                stripe.handleCardPayment(response.payment_intent_client_secret, data).then(function (result) {
                  if (result.error) {
                    ev.complete('fail');
                    $("#stripe-pi-payment-request-button").prepend('<div class="messages error">' + result.error.message + '</div>');
                  }
                  else {
                    //We need to send back info to controller and to add new data
                    // to order and payment
                    fetch(payment_intent_path, {
                      method: 'POST',
                      body: JSON.stringify({data: result}),
                      headers: {'content-type': 'application/json'},
                    })
                      .then(function (response) {
                        response.json().then(function (json) {
                          if (json.error) {
                            ev.complete('fail');
                            $("#stripe-pi-payment-request-button").prepend('<div class="messages error">' + json.error + '</div>');
                          }
                          else if (json.requires_action) {
                            ev.complete('success');
                            handleAction(json, payment_intent_path, ev);
                          }
                          else {
                            ev.complete('success');
                            window.location.href = json.redirect_uri;
                          }
                        });
                      });
                  }
                });
              }
              var payment_intent_path = Drupal.settings.basePath + Drupal.settings.pathPrefix + Drupal.settings.commerce_stripe_pi_payment_request.payment_intent_path;
              // Send the card details to the server so we can charge the card
              // and complete the payment.
              fetch(payment_intent_path, {
                method: 'POST',
                body: JSON.stringify({data: payment_intent_data}),
                headers: {'content-type': 'application/json'},
              })
                .then(function (response) {
                  response.json().then(function (json) {
                    if (json.error) {
                      ev.complete('fail');
                      $("#stripe-pi-payment-request-button").prepend('<div class="messages error">' + json.error + '</div>');
                    }
                    else if (json.requires_action) {
                      ev.complete('success');
                      handleAction(json, payment_intent_path, ev);
                    }
                    else {
                      ev.complete('success');
                      // Redirect to complete page.
                      window.location.href = json.redirect_uri;
                    }
                  });
                });
            });


          });
        }
        // Hides buttons and removes class if Apple/Google Pay tab isn't active.
        else {
          Drupal.behaviors.commerceStripePiWebPaymentRequest.hideButtons();
          $form.removeClass('stripe-commerce-payment-request-not-configured');
        }
      }
    },

    // Show our Apple/Google Pay button, if payments can be made.
    showButton: function () {
       $('.checkout-continue').hide();
      $('.stripe-payment-request-setup-wrapper').show();
      $('#stripe-pi-payment-request-button').show();
    },

    // Show the setup info, if payments can't be made.
    showSetUpButton: function () {
       $('.checkout-continue').hide();
      // Shows wrapper and button, because Stripe hides Setup button by default.
      $('.stripe-payment-request-setup-wrapper').show();
      $('#stripe-payment-request-unavailable').show();
    },

    // Hide our Payment Request button if Stripe Payment Request method isn't
    // active.
    hideButtons: function () {
      $('#stripe-pi-payment-request-button').hide();
      $('.stripe-payment-request-setup-wrapper').hide();
       $('.checkout-continue').show();
    },
    getAddressData: function($address_component) {
      if ($address_component !== "undefined") {
        return $address_component;
      }else{
        return "";
      }
    }
  };
})(jQuery);
